"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseClient = void 0;
const lodash_1 = __importDefault(require("lodash"));
const ms_1 = __importDefault(require("ms"));
const querystring_1 = __importDefault(require("querystring"));
const restify_clients_1 = require("restify-clients");
const url_1 = require("url");
const formatted_error_utils_1 = require("../utils/formatted-error.utils");
const SUPPORTED_METHODS = ['get', 'head', 'post', 'put', 'patch', 'delete'];
const SUPPORTS_BODY = ['post', 'put', 'patch'];
class BaseClient {
    constructor(baseUrl, options) {
        const { pathPrefix, apiOptions } = BaseClient.parseConstructorOptions(baseUrl, options);
        this.baseUrl = baseUrl;
        this.pathPrefix = pathPrefix;
        this.apiOptions = apiOptions;
        this.api = (0, restify_clients_1.createJsonClient)(this.apiOptions);
    }
    apiCall(options, callback) {
        this._checkArgs(options);
        const formattedQs = options.qs ? `?${querystring_1.default.stringify(options.qs)}` : '';
        const opts = {
            path: this.pathPrefix + options.path + formattedQs,
            headers: Object.assign(this._extendedHeaders({ body: options.body, qs: options.qs }), options.headers)
        };
        const args = options.body
            ? [opts, options.body]
            : [opts];
        return this.api[options.method](...args, (err, req, res, obj) => {
            return err
                ? callback(err)
                : callback(null, obj);
        });
    }
    static parseConstructorOptions(baseUrl, options) {
        var _a;
        const parsedUrl = (0, url_1.parse)(baseUrl);
        if (parsedUrl.hash || (parsedUrl.pathname !== parsedUrl.path))
            throw new Error('"Unclean" path is not supported: no query strings, hashes, etc.');
        return {
            pathPrefix: (_a = parsedUrl.pathname) === null || _a === void 0 ? void 0 : _a.replace(/\/+$/g, ''),
            apiOptions: lodash_1.default.merge({ url: (0, url_1.format)({ protocol: parsedUrl.protocol, hostname: parsedUrl.hostname, port: parsedUrl.port }) }, BaseClient.defaultOptions, options)
        };
    }
    _checkArgs(options) {
        if (!SUPPORTED_METHODS.includes(options.method))
            throw new formatted_error_utils_1.FormattedError('`method` argument must be one of `%j`, got `%j`', SUPPORTED_METHODS, options.method);
        if ((typeof options.path !== 'string') || (options.path.length < 1))
            throw new formatted_error_utils_1.FormattedError('`path` argument must be non-empty string, got `%j`', options.path);
        if (!SUPPORTS_BODY.includes(options.method) && (options.body !== null))
            throw new formatted_error_utils_1.FormattedError('`%s` does not support body: `%j` passed in, expected `null` (use `qs` param instead)', options.method, options.body);
    }
    _extendedHeaders(options) {
        const reqId = (options.qs || {}).req_id || (options.body || {}).req_id || null;
        return reqId
            ? { 'x-request-id': reqId }
            : {};
    }
}
exports.BaseClient = BaseClient;
BaseClient.defaultOptions = {
    // Enable retries in establishing TCP connection
    // (this will not retry on HTTP errors).
    retry: {
        minTimeout: (0, ms_1.default)('1 second'),
        maxTimeout: (0, ms_1.default)('5 seconds'),
        retries: 3
    },
    headers: {
        'accept': 'application/json',
        'accept-encoding': 'gzip,deflate'
    }
};
